<?php

/**
 * @file
 * Flexible style plugin.
 */

$plugin = array(
  'title' => t('Flexible'),
  'description' => t('Add full-width support.'),
  'render pane' => 'radix_full_width_flexible_render_pane',
  'render region' => 'radix_full_width_flexible_render_region',
  'settings form' => 'radix_full_width_flexible_settings_form',
  'settings form validate' => 'radix_full_width_flexible_settings_validate',
  'pane settings form' => 'radix_full_width_flexible_settings_form',
  'pane settings form validate' => 'radix_full_width_flexible_settings_validate',

  // Signals to radix_full_width that we can work as in 'full width' mode
  // if instructed.
  'full width' => TRUE,
);

/**
 * Settings form callback.
 */
function radix_full_width_flexible_settings_form($settings) {
  $form = array();
  
  //
  // Settings connected with size.
  //

  $form['size'] = array(
    '#type' => 'fieldset',
    '#title' => t('Size'),
    '#parents' => array('settings'),
  );

  $form['size']['min_height'] = array(
    '#type' => 'textfield',
    '#title' => t('Minimum height'),
    '#default_value' => isset($settings['min_height']) ? $settings['min_height'] : '0',
    '#field_suffix' => 'px',
  );

  $padding_types = array(
    'padding_top' => t('Padding top'),
    'padding_left' => t('Padding left'),
    'padding_bottom' => t('Padding bottom'),
    'padding_right' => t('Padding right'),
  );
  foreach ($padding_types as $padding_type => $padding_label) {
    $form['size'][$padding_type] = array(
      '#type' => 'textfield',
      '#title' => $padding_label,
      '#default_value' => isset($settings[$padding_type]) ? $settings[$padding_type] : '0',
      '#field_suffix' => 'px',
    );
  }

  $form['size']['full_width_content'] = array(
    '#type' => 'checkbox',
    '#title' => t('Make content of region full width'),
    '#description' => t('This will only work if this layout supports it and it renders the full page.'),
    '#default_value' => isset($settings['full_width_content']) ? $settings['full_width_content'] : FALSE,
  );

  return $form;
}

/**
 * Settings validate callback.
 */
function radix_full_width_flexible_settings_validate($form, $settings) {
  // Check all the integer fields.
  $integer_fields = array(
    'padding_top',
    'padding_left',
    'padding_bottom',
    'padding_right',
  );
  foreach ($integer_fields as $integer_field) {
    if ((int)$settings[$integer_field] != $settings[$integer_field] || $settings[$integer_field] < 0) {
      form_set_error('settings][' . $integer_field, t('Must be a positive, whole number.'));
    }
  }
}

/**
 * Generate CSS classes or attributes for the given settings.
 */
function _radix_full_width_flexible_build_attributes($settings, $type) {
  $class = array($type . '-radix-full-width-flexible');
  $style = array();

  // Size settings.
  if (!empty($settings['min_height'])) {
    $style['min-height'] = $settings['min_height'] . 'px';
  }
  if (!empty($settings['padding_top'])) {
    $style['padding-top'] = $settings['padding_top'] . 'px';
  }
  if (!empty($settings['padding_left'])) {
    $style['padding-left'] = $settings['padding_left'] . 'px';
  }
  if (!empty($settings['padding_bottom'])) {
    $style['padding-bottom'] = $settings['padding_bottom'] . 'px';
  }
  if (!empty($settings['padding_right'])) {
    $style['padding-right'] = $settings['padding_right'] . 'px';
  }

  // Convert into an array.
  $attributes = array();
  if (!empty($class)) {
    $attributes['class'] = $class;
  }
  if (!empty($style)) {
    $attributes['style'] = $style;
  }
  return $attributes;
}

/**
 * Collapse an associative style array into a simple array.
 */
function _radix_full_width_flexible_style_array($style) {
  $style_array = array();
  foreach ($style as $style_name => $style_value) {
     $style_array[] = "$style_name: $style_value;";
  }
  return $style_array;
}

/**
 * Take an attributes array and generate the string.
 */
function _radix_full_width_flexible_render_attributes($attributes) {
  // Collapse style into an array.
  if (!empty($attributes['style'])) {
    $attributes['style'] = _radix_full_width_flexible_style_array($attributes['style']);
  }
  return drupal_attributes($attributes);
}

/**
 * Theme function for the pane style.
 */
function theme_radix_full_width_flexible_render_pane($vars) {
  $content = $vars['content'];
  $pane = $vars['pane'];
  $display = $vars['display'];
  $plugin = $vars['style'];
  $settings = $vars['settings'];

  $attributes = _radix_full_width_flexible_build_attributes($settings, 'pane');
  if (!empty($attributes['class'])) {
    $content->css_class = (!empty($content->css_class) ? $content->css_class . ' ' : '') . implode(' ', $attributes['class']);
  }
  if (!empty($attributes['style'])) {
    $content->style_attribute = implode(' ', _radix_full_width_flexible_style_array($attributes['style']));
  }
  
  // if the style is gone or has no theme of its own, just display the output.
  return theme('panels_pane', array('content' => $content, 'pane' => $pane, 'display' => $display));
}

/**
 * Theme function for the region style.
 */
function theme_radix_full_width_flexible_render_region($vars) {
  $display = $vars['display'];
  $region_id = $vars['region_id'];
  $panes = $vars['panes'];
  $settings = $vars['settings'];

  $add_container = $vars['full_width'] && empty($settings['full_width_content']) && radix_full_width_is_region_full_width($vars['region_id'], $vars['display']->layout);

  $attributes = _radix_full_width_flexible_build_attributes($settings, 'region');

  // If adding a container, move some attributes to the container.
  $container_attributes = array();
  if ($add_container) {
    foreach ($attributes['style'] as $style_name => $style_value) {
      // Move all the padding attributes.
      if (strpos($style_name, 'padding') === 0) {
        $container_attributes['style'][$style_name] = $style_value;
        unset($attributes['style'][$style_name]);
      }
    }
  }

  // Add our default classes.
  if (empty($attributes['class'])) {
    $attributes['class'] = array();
  }
  $attributes['class'] = array_merge(
    array('region', 'region-' . $vars['region_id']),
    $attributes['class']);

  $output = '<div' . _radix_full_width_flexible_render_attributes($attributes) . '>';

  if ($add_container) {
    $output .= '<div class="container">';
    if (!empty($container_attributes)) {
      $output .= '<div class="container-inner"' . _radix_full_width_flexible_render_attributes($container_attributes) . '>';
    }
  }
  $output .= implode('<div class="panel-separator"></div>', $panes);
  if ($add_container) {
    $output .= '</div>';
    if (!empty($container_attributes)) {
      $output .= '</div>';
    }
  }

  $output .= '</div>';

  return $output;
}
